from typing import Dict, Callable

from petisco.events.event import Event
from petisco.events.interface_event_manager import IEventManager


class FakeEventManager(IEventManager):

    is_subscribed = False

    def __init__(self, subscribers: Dict[str, Callable]):
        super().__init__(subscribers)
        if subscribers:
            self.is_subscribed = True

    def unsubscribe_all(self):
        self.is_subscribed = False

    def send(self, topic: str, event: Event):
        if self.is_subscribed and topic in self.subscribers:
            self.subscribers.get(topic)(event)



from petisco.events.event import Event


def redis_based_event_handler(func):
    def wrapper(message):
        json_data = message["data"].decode("utf-8")
        event = Event.from_json(json_data)
        return func(event)

    return wrapper



import re
from meiga import Result, Error, Failure

from petisco.domain.errors.given_input_is_not_valid_error import (
    GivenInputIsNotValidError,
)
from petisco.domain.errors.input_exceed_lenght_limit_error import (
    InputExceedLengthLimitError,
)

NAME_LENGTH_LIMIT = 50


class Name(str):
    def __new__(cls, name, length_limit: int = 50):
        name = None if name == "None" else name
        cls.length_limit = length_limit
        return str.__new__(cls, name)

    def handle(self) -> Result[bool, Error]:
        if self is not None:
            if len(self) > self.length_limit:
                return Failure(InputExceedLengthLimitError(message=self))
            else:
                if not re.search(r"^[a-zA-Z]+(([',. -][a-zA-Z ])?[a-zA-Z]*)*$", self):
                    return Failure(GivenInputIsNotValidError(message=self))
        return self



from datetime import datetime
from typing import Dict, Any

from dataclasses import dataclass, field
from dataclasses_json import dataclass_json, config
from marshmallow import fields
from backports.datetime_fromisoformat import MonkeyPatch

from petisco.events.event_id import EventId

MonkeyPatch.patch_fromisoformat()


@dataclass_json
@dataclass
class Event:
    event_id: EventId = None
    event_name: str = None
    occurred_on: datetime = field(
        default=None,
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format="iso"),
        ),
    )
    data: Dict[str, Any] = None

    def __init__(self, *args, **kwargs):
        self.occurred_on = kwargs.get("occurred_on", datetime.utcnow())
        self.data = kwargs
        self.event_id = EventId.generate(str(kwargs))
        self.event_name = self.get_event_name()

    @classmethod
    def get_event_name(cls):
        return "{}.{}".format(cls.__module__, cls.__name__)
